# Generated by roxygen2: do not edit by hand

S3method(contrib,default)
S3method(contrib,pcss.core)
S3method(coreplot,default)
S3method(coreplot,pcss.core)
S3method(print,pcss.core)
S3method(rpcss::biplot,pcss.core)
S3method(rpcss::screeplot,pcss.core)
S3method(rpcss::subset,pcss.core)
export(contrib)
export(coreplot)
export(pcss.core)
import(ggplot2)
import(gslnls)
import(mathjaxr)
importFrom(FactoMineR,CA)
importFrom(FactoMineR,FAMD)
importFrom(FactoMineR,PCA)
importFrom(Rdpack,reprompt)
importFrom(dplyr,bind_rows)
importFrom(ggrepel,geom_label_repel)
importFrom(methods,is)
importFrom(stats,biplot)
importFrom(stats,coef)
importFrom(stats,screeplot)
importFrom(tidyr,pivot_longer)
importFrom(utils,combn)
importFrom(utils,data)
